//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

//#include "IvoryDraw.h"
//---------------------------------------------------------------------------
__fastcall TIvoryCanvas::TIvoryCanvas()
 : FHandle(NULL), FOnChanging(NULL), FOnChange(NULL)
{
	FPen = new TPen();
	FPen->OnChange = PenChanged;
   FBrush = new TBrush();
   FBrush->OnChange = BrushChanged;
   FFont = new TFont();
   FFont->OnChange = FontChanged;
}
//---------------------------------------------------------------------------
__fastcall TIvoryCanvas::~TIvoryCanvas()
{
	FPen->Free();
   FBrush->Free();
   FFont->Free();
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::SetHandle(HDC handle)
{
	FHandle = handle;
   PenChanged(FPen);
   BrushChanged(FBrush);
   FontChanged(FFont);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::SetPen(TPen* pen)
{
	FPen->Assign(pen);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::SetBrush(TBrush* brush)
{
	FBrush->Assign(brush);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::SetFont(TFont* font)
{
	FFont->Assign(font);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::PenChanged(TObject* pen)
{
	if(Handle && ((TPen*)pen)->Handle)
		SelectObject(Handle, ((TPen*)pen)->Handle);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::BrushChanged(TObject* brush)
{
	if(Handle && ((TBrush*)brush)->Handle)
   {
		SelectObject(Handle, ((TBrush*)brush)->Handle);
      SetBkColor(Handle, Brush->Color);
   }
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::FontChanged(TObject* font)
{
	if(Handle && ((TFont*)font)->Handle)
   {
		SelectObject(Handle, ((TFont*)font)->Handle);
      SetTextColor(Handle, ((TFont*)font)->Color);
   }
}
//---------------------------------------------------------------------------
// draw a bitmap on the canvas
void __fastcall TIvoryCanvas::Draw(int x, int y, TIvoryBitmap* bmp)
{
	if(bmp)
   	Draw(x, y, bmp->Handle);
}
//---------------------------------------------------------------------------
// draw a bitmap on the canvas
void __fastcall TIvoryCanvas::Draw(int x, int y, HBITMAP bmp)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Draw_Handle");
	if(bmp)
   {
   	BITMAP bm;
      GetObject(bmp, sizeof(bm), &bm);
      HDC hdcImage = CreateCompatibleDC(NULL);
      SelectObject(hdcImage, bmp);
      BitBlt(Handle, x, y, bm.bmWidth, bm.bmHeight, hdcImage, 0, 0, SRCCOPY);
      DeleteDC(hdcImage);
   }
}
//---------------------------------------------------------------------------
// draw a bitmap stretched on the canvas
void __fastcall TIvoryCanvas::StretchDraw(const Windows::TRect& Rect, TIvoryBitmap* bmp)
{
	if(bmp)
   	StretchDraw(Rect, bmp->Handle);
}
//---------------------------------------------------------------------------
// draw a bitmap stretched on the canvas
void __fastcall TIvoryCanvas::StretchDraw(const Windows::TRect& Rect, HBITMAP bmp)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::StretchDraw_Handle");
	if(bmp)
   {
   	BITMAP bm;
      GetObject(bmp, sizeof(bm), &bm);
      HDC hdcImage = CreateCompatibleDC(NULL);
      SelectObject(hdcImage, bmp);
      StretchBlt(Handle, Rect.Left, Rect.Top, Rect.Right-Rect.Left, Rect.Bottom-Rect.Top, hdcImage, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY);
      DeleteDC(hdcImage);
   }
}
//---------------------------------------------------------------------------
// draw a bitmap tiled on the canvas
void __fastcall TIvoryCanvas::TileDraw(const Windows::TRect& Rect, TIvoryBitmap* bmp)
{
	if(bmp)
   	TileDraw(Rect, bmp->Handle);
}
//---------------------------------------------------------------------------
// draw a bitmap tiled on the canvas
void __fastcall TIvoryCanvas::TileDraw(const Windows::TRect& Rect, HBITMAP bmp)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::TileDraw_Handle");
	if(bmp)
   {
   	int x = Rect.Left, y = Rect.Top, width = abs(Rect.Right-Rect.Left), height = abs(Rect.Bottom-Rect.Top);
   	BITMAP bm;
      GetObject(bmp, sizeof(bm), &bm);
      if(bm.bmHeight == 0 || bm.bmWidth ==0)
         return;
      HDC hdcImage = CreateCompatibleDC(NULL);
      SelectObject(hdcImage, bmp);
      int horizN = width / bm.bmWidth;
      int horizR = width - horizN * bm.bmWidth;
      int vertN = height / bm.bmHeight;
      int vertR = height - vertN * bm.bmHeight;
      int currX, currY;

      for(currY = y; currY < vertN * bm.bmHeight; currY += bm.bmHeight)
      {
	      for(currX = x; currX < horizN * bm.bmWidth; currX += bm.bmWidth)
		      BitBlt(Handle, currX, currY, bm.bmWidth, bm.bmHeight, hdcImage, 0, 0, SRCCOPY);
			if(horizR)
		      BitBlt(Handle, currX, currY, horizR, bm.bmHeight, hdcImage, 0, 0, SRCCOPY);
      }
      if(vertR)
      {
	      for(currX = x; currX < horizN * bm.bmWidth; currX += bm.bmWidth)
		      BitBlt(Handle, currX, currY, bm.bmWidth, vertR, hdcImage, 0, 0, SRCCOPY);
         if(horizR)
		      BitBlt(Handle, currX, currY, horizR, vertR, hdcImage, 0, 0, SRCCOPY);
      }
      DeleteDC(hdcImage);
   }
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::Rectangle(int x1, int y1, int x2, int y2)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Rectangle_Handle");
	if(OnChanging) OnChanging(this);
	::Rectangle(Handle, x1, y1, x2, y2);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::Arc(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Arc_Handle");
	if(OnChanging) OnChanging(this);
	::Arc(Handle, x1, y1, x2, y2, x3, y3, x4, y4);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::Chord(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Chord_Handle");
	if(OnChanging) OnChanging(this);
	::Chord(Handle, x1, y1, x2, y2, x3, y3, x4, y4);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::DrawFocusRect(const TRect rect)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::DrawFocusRect_Handle");
	if(OnChanging) OnChanging(this);
   ::DrawFocusRect(Handle, (CONST RECT*)&rect);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::Ellipse(int x1, int y1, int x2, int y2)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Ellipse_Handle");
	if(OnChanging) OnChanging(this);
   ::Ellipse(Handle, x1, y1, x2, y2);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::FillRect(const TRect rect)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::FillRect_Handle");
	if(OnChanging) OnChanging(this);
   ::FillRect(Handle, (CONST RECT*)&rect, Brush->Handle);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::FloodFill(int x, int y, TColor color, TFillStyle fillStyle)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::FloodFill_Handle");
	int FillStyles[] = { FLOODFILLSURFACE, FLOODFILLBORDER };
	if(OnChanging) OnChanging(this);
   ::ExtFloodFill(Handle, x, y, color, FillStyles[fillStyle]);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::FrameRect(const TRect rect)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::FrameRect_Handle");
	if(OnChanging) OnChanging(this);
   ::FrameRect(Handle, (CONST RECT*)&rect, Brush->Handle);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::LineTo(int x, int y)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::LineTo_Handle");
	if(OnChanging) OnChanging(this);
   ::LineTo(Handle, x, y);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::MoveTo(int x, int y)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::MoveTo_Handle");
   ::MoveToEx(Handle, x, y, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::Pie(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::Pie_Handle");
	if(OnChanging) OnChanging(this);
	::Pie(Handle, x1, y1, x2, y2, x3, y3, x4, y4);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::RoundRect(int x1, int y1, int x2, int y2, int x3, int y3)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::RoundRect_Handle");
	if(OnChanging) OnChanging(this);
	::RoundRect(Handle, x1, y1, x2, y2, x3, y3);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
int __fastcall TIvoryCanvas::TextHeight(const String text)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::TextHeight_Handle");
	TSize Extent;
   if(::GetTextExtentPoint(Handle, text.c_str(), text.Length(), &Extent))
		return Extent.cy;
   else
   	return 0;
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::TextOut(int x, int y, const String text)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::TextOut_Handle");
	if(OnChanging) OnChanging(this);
   ::TextOut(Handle, x, y, text.c_str(), text.Length());
   MoveTo(x + TextWidth(text), y);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TIvoryCanvas::TextRect(TRect rect, int x, int y, const String text)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::TextRect_Handle");
	int Options = ETO_CLIPPED;
	if(OnChanging) OnChanging(this);
	if(Brush->Style != bsClear) Options |= ETO_OPAQUE;
   ::ExtTextOut(Handle, x, y, Options, (CONST RECT*)&rect, text.c_str(), text.Length(), NULL);
   if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
int __fastcall TIvoryCanvas::TextWidth(const String text)
{
	if(!Handle) throw EIvoryError(iecCanvasError, "TIvoryCanvas::TextWidth_Handle");
	TSize Extent;
   if(::GetTextExtentPoint(Handle, text.c_str(), text.Length(), &Extent))
		return Extent.cx;
   else
   	return 0;
}
//---------------------------------------------------------------------------

